package org.expeditee.settings;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.expeditee.agents.SearchGreenstone;
import org.expeditee.agents.mail.MailSession;
import org.expeditee.agents.wordprocessing.JSpellChecker;
import org.expeditee.gui.AttributeValuePair;
import org.expeditee.gui.DisplayIO;
import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameIO;
import org.expeditee.gui.FrameUtils;
import org.expeditee.gui.FreeItems;
import org.expeditee.gui.MessageBay;
import org.expeditee.gui.Reminders;
import org.expeditee.items.Item;
import org.expeditee.items.ItemUtils;
import org.expeditee.items.Text;

/**
 * Central class to contain all values that can be set by the user on their
 * profile frame. These values should be updated only by
 * FrameUtils.ParseProfile.
 */
public abstract class UserSettings {
	
	public final static String DEFAULT_PROFILE_NAME = "default";
	
	/*
	 * General settings (no setter functions)
	 */
	public static final int defaultGravity = 3;
	public static int Gravity = defaultGravity;
	
	public static final float defaultScaleFactor = 1F;
	public static float ScaleFactor = defaultScaleFactor;

	public static final int defaultLineStraightenThreshold = 15;
	public static int LineStraightenThreshold = defaultLineStraightenThreshold;

	public static final int defaultNoOpThreshold = 60;
	public static int NoOpThreshold = defaultNoOpThreshold;
	
	public static final int defaultTitlePosition = 150;
	public static int TitlePosition = defaultTitlePosition;

	public static final int defaultInitialWidth = 1024;
	public static int InitialWidth = defaultInitialWidth;
	
	public static final int defaultInitialHeight = 768;
	public static int InitialHeight = defaultInitialHeight;
	
	public static String ProfileName;
	
	public static String UserName;
	
	public static final boolean defaultAntiAlias = false;
	public static boolean AntiAlias = defaultAntiAlias;

	public static final boolean defaultLineHighlight = false;
	public static boolean LineHighlight = defaultLineHighlight;

	public static final boolean defaultLogging = false;
	public static boolean Logging = defaultLogging;

	public static final boolean defaultLogStats = true;
	public static boolean LogStats = defaultLogStats;

	public static final boolean defaultThreading = true;
	public static boolean Threading = defaultThreading;
	
	public static final boolean defaultAutoWrap = false;
	public static boolean AutoWrap = defaultAutoWrap;
	
	
	/*
	 * Frames
	 */
	public static String DefaultFrame = null;
	public static void setDefaultFrame(Text t) {
		UserSettings.DefaultFrame = FrameUtils.getLink(t, UserSettings.DefaultFrame);
	}
	
	public static String StatisticsFrameset = null;

	public static String MenuFrame = null;

	public static String HomeFrame = null;
	public static void setHomeFrame(Text t) {
		String first = FrameUtils.getLink(t, UserSettings.HomeFrame);
		// do not use non-existant frames as the first frame
		if (FrameIO.isValidFrameName(first)) {
			UserSettings.HomeFrame = first;
		}
		// warn the user
		else {
			// MessageBay.warningMessage("Home frame: " + first
			// + " is not a valid frame.");
			UserSettings.HomeFrame = FrameIO.LoadProfile(UserSettings.ProfileName).getName();
		}
	}

	/*
	 *  Directories
	 */
	public static List<String> FrameDirs = new LinkedList<String>();
	public static void setFrameDirs(Text t) {
		UserSettings.FrameDirs.addAll(FrameUtils.getDirs(t));
	}
	public static void setFramesetDir(Text t) {
		String dir = FrameUtils.getDir(new AttributeValuePair(t.getText()).getValue());
		if (dir != null) {
			UserSettings.FrameDirs.add(dir);
		}
	}

	public static List<String> ImageDirs = new LinkedList<String>();
	public static void setImageDirs(Text t) {
		UserSettings.ImageDirs.addAll(FrameUtils.getDirs(t));
	}
	public static void setImageDir(Text t) {
		String dir = FrameUtils.getDir(new AttributeValuePair(t.getText()).getValue());
		if (dir != null) {
			UserSettings.ImageDirs.add(0, dir);
		}
	}
	
	public static void setLogDir(Text t) {
		org.expeditee.gui.FrameIO.LOGS_DIR = FrameUtils.getDir(new AttributeValuePair(t.getText()).getValue());
	}

	/*
	 *  Templates
	 */
	public static Text TitleTemplate = null;
	
	public static Text ItemTemplate = new Text("@ItemTemplate");
	public static void setItemTemplate(Text t) {
		UserSettings.ItemTemplate = t.getTemplateForm();
	}
	
	public static Text DotTemplate = new Text("@DotTemplate");
	public static void setDotTemplate(Text t) {
		UserSettings.DotTemplate = t.getTemplateForm();
	}

	public static Text AnnotationTemplate = null;
	public static void setAnnotationTemplate(Text t) {
		UserSettings.AnnotationTemplate = t.getTemplateForm();
	}

	public static Text CodeCommentTemplate = null;
	public static void setCommentTemplate(Text t) {
		UserSettings.CodeCommentTemplate = t.getTemplateForm();
	}

	public static Text StatTemplate = null;
	public static void setStatsTemplate(Text t) {
		UserSettings.StatTemplate = t.getTemplateForm();
	}
	
	/*
	 * Colorwheels
	 */
	public static void setColorWheel(Text t) {
		Item.COLOR_WHEEL = FrameUtils.getColorWheel(t);
	}
	
	public static void setFillColorWheel(Text t) {
		Item.FILL_COLOR_WHEEL = FrameUtils.getColorWheel(t);
	}
	
	public static void setBackgroundColorWheel(Text t) {
		Frame.COLOR_WHEEL = FrameUtils.getColorWheel(t);
	}
	
	/*
	 * Other
	 */
	public static List<Text> Style = new LinkedList<Text>();
	public static void setStyle(Text t) {
		Frame child = t.getChild();
		if (child == null)
			UserSettings.Style = new LinkedList<Text>();

		List<Text> style = new ArrayList<Text>(8);
		for (int i = 0; i < 10; i++) {
			style.add(null);
		}

		for (Text text : child.getBodyTextItems(false)) {
			String type = text.getText();
			char lastChar = type.charAt(type.length() - 1);
			if (Character.isDigit(lastChar)) {
				style.set(lastChar - '0', text);
			} else {
				style.set(0, text);
			}
		}
		UserSettings.Style = style;
	}
	
	public static void setSpellChecker(Text t) {
		try {
			JSpellChecker.create();
		} catch (FileNotFoundException e) {
			MessageBay.errorMessage("Could not find dictionary: "
					+ e.getMessage());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	public static void setSpelling(Text t) {
		try {
	        JSpellChecker.create(t.getChild());
        } catch (Exception e) {
	        e.printStackTrace();
        }
	}
	
	public static void setGreenstoneSettings(Text t) {
		SearchGreenstone.init(t.getChild());
	}
	
	public static void setReminders(Text t) {
		Reminders.init(t.getChild());
	}
	
	public static void setFormatSpacingMin(Text t) {
		FrameUtils.MINIMUM_SPACING_RATIO = new AttributeValuePair(t.getText()).getDoubleValue();
	}
	
	public static void setFormatSpacingMax(Text t) {
		FrameUtils.MAXIMUM_SPACING_RATIO = new AttributeValuePair(t.getText()).getDoubleValue();
	}
	
	public static void setMailSettings(Text t) {
		MailSession.init(t.getChild());
	}
	
	public static void setCursorFrame(Text t) {
		if (t.getChild() != null) {
			FreeItems.getCursor().addAll(ItemUtils.CopyItems(t.getChild().getAllItems()));
			for (Item i : FreeItems.getCursor()) {
				i.setParent(null);
			}
			DisplayIO.setCursor(Item.HIDDEN_CURSOR);
			DisplayIO.setCursor(Item.DEFAULT_CURSOR);
		}
	}
	
	
	// add default values
	static {
		String expeditee_home = System.getProperty("expeditee.home");
		if (expeditee_home != null) {
            FrameIO.changeParentFolder(expeditee_home + File.separator);
        } else {
        	FrameIO.changeParentFolder(getSaveLocation());
        }
	  
		UserSettings.FrameDirs.add(FrameIO.FRAME_PATH);
		UserSettings.FrameDirs.add(FrameIO.PUBLIC_PATH);
		UserSettings.FrameDirs.add(FrameIO.PROFILE_PATH);
		UserSettings.FrameDirs.add(FrameIO.HELP_PATH);
		UserSettings.ImageDirs.add(FrameIO.IMAGES_PATH);
		UserSettings.FrameDirs.add(FrameIO.MESSAGES_PATH);
	}

	/**
	 * Find the appropriate directory to store application settings in for
	 * the current OS.
	 * This has only been tested on Linux so far, so if it doesn't work it
	 * may need to be modified or reverted. Should return:
	 * 	Linux: ~/.expeditee
	 * 	Windows: %appdata%\.expeditee
	 * 	Mac: ~/Library/Application\ Support/.expeditee
	 * @return the path to store expeditee's settings
	 */
	public static String getSaveLocation() {
		String OS=System.getProperty("os.name").toLowerCase();
		if(OS.indexOf("win")>0) {			//windoze
			return System.getenv("APPDATA")+File.separator+".expeditee"+File.separator;
		} else if(OS.indexOf("mac")>0) {	//mac
			return System.getProperty("user.home")+File.separator+"Library"+File.separator+"Application Support"+File.separator+".expeditee"+File.separator;
		} else {							//linux or other
			return System.getProperty("user.home")+File.separator+".expeditee"+File.separator;
		}
	}
}
